@@ -1,4 +1,4 @@
-# Copyright 2016 - 2020  Ternaris.
+# Copyright 2016 - 2021  Ternaris.
 # SPDX-License-Identifier: AGPL-3.0-only
 
 # pylint: disable=too-many-lines
@@ -18,7 +18,7 @@ from logging import getLogger
 from pathlib import Path
 
 import bcrypt
-from pypika import JoinType, Order
+from pypika import JoinType, NullValue, Order
 from pypika import SQLLiteQuery as Query
 from pypika import Table, Tables, Tuple
 from pypika import functions as fn
@@ -39,6 +39,19 @@ log = getLogger(__name__)
 
 USERGROUP_REGEX = re.compile(r'[0-9a-zA-Z\-_\.@+]+$')
 
+# Order corresponds to marv.model.STATUS OrderedDict
+STATUS_ICON = ['fire', 'eye-close', 'warning-sign', 'time']
+STATUS_JSON = [json.dumps({'icon': STATUS_ICON[i], 'title': x},
+                          separators=(',', ':'))
+               for i, x in enumerate(STATUS.values())]
+# TODO: reconsider in case we get a couple of more states
+STATUS_STRS = {
+    bitmask: ','.join(filter(None,
+                             (STATUS_JSON[i] if bitmask & 2**i else None
+                              for i in range(len(STATUS_JSON)))))
+    for bitmask in range(2**(len(STATUS_JSON)))
+}
+
 
 class NoSetidFound(Exception):
     pass
@@ -154,7 +167,10 @@ class FromExceptQuery:
 
 class ValuesTuple(Tuple):
     def get_sql(self, **kw):
-        return f'(VALUES {",".join(x.get_sql(**kw) for x in self.values)})'
+        sql = f'(VALUES {",".join(x.get_sql(**kw) for x in self.values)})'
+        if self.alias:
+            return f'{sql} AS {self.alias}'
+        return sql
 
 
 class GroupConcat(AggregateFunction):
@@ -303,7 +319,7 @@ def resolve_filter(table, fltr, models):  # noqa: C901  pylint: disable=too-many
 
 def cleanup_attrs(items):
     return [
-        {k: x[k] for k in x.keys() if k not in ['acn_id', 'password', 'row']}
+        {k: x[k] for k in x.keys() if k not in ['acn_id', 'dacn_id', 'password', 'row']}
         for x in items
     ]
 
@@ -395,7 +411,7 @@ async def dump_users_groups(tables, dump, txn):
     dump['users'] = users = []
     items = await get_items_for_query(Query.from_(user_t)
                                       .select('*')
-                                      .where(~escaped_startswith(user_t.name, 'marv:'))
+                                      .where(user_t.name != 'marv:anonymous')
                                       .orderby(user_t.name),
                                       txn)
     for user in items:
@@ -465,6 +481,7 @@ async def dump_datasets(tables, dump, txn):
         did = dataset.pop('id')
         del dataset['acn_id']
         del dataset['collection_id']
+        del dataset['dacn_id']
         dataset['comments'] = comments.pop(did, [])
         dataset['files'] = files.pop(did)
         dataset['tags'] = tags.pop(did, [])
@@ -508,10 +525,21 @@ class Tortoise(_Tortoise):
         return [model]
 
 
+ACLS = {
+    'comment': ['__authenticated__'],
+    'delete': ['admin'],
+    'download_raw': ['__authenticated__'],
+    'list': ['__authenticated__'],
+    'read': ['__authenticated__'],
+    'tag': ['__authenticated__'],
+}
+
+
 class Database:
     # pylint: disable=too-many-public-methods
 
-    VERSION = '20.12'
+    VERSION = '21.05'
+    DUMP_VERSION = '21.05'
 
     EXPORT_HANDLERS = (
         ({'group', 'user', 'user_group'}, dump_users_groups),
@@ -530,10 +558,18 @@ class Database:
 
     MODELS = MODELS
 
-    def __init__(self, listing_models):
+    def __init__(self, listing_models, config):
         self.connections = []
         self.connection_queue = asyncio.Queue()
         self.listing_models = listing_models
+        self.config = config
+        self.acls = ACLS.copy()
+        if self.config.marv.ce_anonymous_readonly_access:
+            self.acls.update({
+                'download_raw': ['__unauthenticated__', '__authenticated__'],
+                'list': ['__unauthenticated__', '__authenticated__'],
+                'read': ['__unauthenticated__', '__authenticated__'],
+            })
 
     async def initialize_connections(self):
         defcon = Tortoise.get_connection('default')
@@ -567,10 +603,10 @@ class Database:
         if not username or not password:
             return False
         try:
-            user = await User.get(name=username, realm='marv').using_db(txn)
+            user = await User.get(name=username, active=True, realm='marv').using_db(txn)
         except DoesNotExist:
             return False
-        return bcrypt.checkpw(password.encode('utf-8'), user.password.encode('utf-8'))
+        return bcrypt.checkpw(password.encode(), user.password.encode())
 
     @run_in_transaction
     async def bulk_um(self, users_add, users_remove, groups_add, groups_remove,  # noqa: C901
@@ -589,7 +625,7 @@ class Database:
             for name in users_add:
                 if not USERGROUP_REGEX.match(name):
                     raise DBError('User name can only contain alphanumeric characters and [-_+@.]')
-                user = await User.create(name=name, realm='marv', using_db=txn)
+                user = await User.create(name=name, realm='marv', active=True, using_db=txn)
                 await user.groups.add(
                     await Group.create(name=f'marv:user:{name}', using_db=txn),
                     everybody,
@@ -617,23 +653,27 @@ class Database:
                 await group.users.add(*users, using_db=txn)
         except DoesNotExist:
             raise DBError('Entity does not exist')
+        except IntegrityError:
+            raise DBError('Entity exists already')
 
     @run_in_transaction
     async def user_add(self, name, password, realm, realmuid, given_name=None, family_name=None,
-                       email=None, time_created=None, time_updated=None, _restore=None, txn=None):
+                       email=None, active=True, time_created=None, time_updated=None, _restore=None,
+                       txn=None):
         # pylint: disable=too-many-arguments
-        if not USERGROUP_REGEX.match(name):
+        if not USERGROUP_REGEX.match(name) and not _restore:
             raise DBError('User name can only contain alphanumeric characters and [-_+@.]')
         if not _restore and password is not None:
-            password = bcrypt.hashpw(password.encode('utf-8'), bcrypt.gensalt()).decode('utf-8')
+            password = bcrypt.hashpw(password.encode(), bcrypt.gensalt()).decode()
         now = datetime.utcnow()  # noqa: DTZ
         time_created = datetime.utcfromtimestamp(time_created) if time_created else now
         time_updated = datetime.utcfromtimestamp(time_updated) if time_updated else now
         try:
             user = await User.create(name=name, password=password, realm=realm,
                                      given_name=given_name, family_name=family_name,
-                                     email=email, realmuid=realmuid, time_created=time_created,
-                                     time_updated=time_updated, using_db=txn)
+                                     email=email, realmuid=realmuid, active=active,
+                                     time_created=time_created, time_updated=time_updated,
+                                     using_db=txn)
             if name != 'marv:anonymous':
                 everybody = await Group.get(name='marv:users').using_db(txn)
                 await user.groups.add(
@@ -671,7 +711,7 @@ class Database:
         except DoesNotExist:
             raise ValueError(f'User {username} does not exist')
 
-        user.password = bcrypt.hashpw(password.encode('utf-8'), bcrypt.gensalt()).decode('utf-8')
+        user.password = bcrypt.hashpw(password.encode(), bcrypt.gensalt()).decode()
         user.time_updated = int(utils.now())
         await user.save(using_db=txn)
 
@@ -750,33 +790,64 @@ class Database:
         return default
 
     @staticmethod
-    def get_actionable(modelname, user, action):
-        # pylint: disable=unused-argument
-        model = Table(modelname)
-        return Query.from_(model).select(model.id)
+    def if_admin(username, query):
+        user, user_group, group = Tables('user', 'user_group', 'group')
+        return (query
+                .join(user)
+                .cross()
+                .join(user_group)
+                .on(user.id == user_group.user_id)
+                .join(group)
+                .on(user_group.group_id == group.id)
+                .where((group.name == 'admin') & (user.name == username)))
 
-    @staticmethod
-    def get_resolve_value_query(rel, relations, user, action):
+    def generate_crit(self, ids, empty, user, action):
+        if user == '::':
+            return ids
+        groups = self.acls[action]
+        if user == 'marv:anonymous':
+            if '__unauthenticated__' in groups:
+                return ids
+            return empty
+
+        if '__authenticated__' in groups:
+            return ids
+
+        return self.if_admin(user, ids)
+
+    def get_actionable(self, modelname, user, action):
+        model = Table(modelname)
+        query = Query.from_(model).select(model.id)
+        return self.generate_crit(query, query.where(model.id.isnull()), user, action)
+
+    def get_resolve_value_query(self, rel, relations, user, action):
         # pylint: disable=unused-argument
         out, tmp = Tables('out', 'tmp')
-        return Query.with_(Query.with_(Query.from_(ValuesTuple(*relations))
-                                       .select('*'), 'tmp(value, back_id)')
-                           .from_(tmp)
-                           .join(rel)
-                           .on(tmp.value == rel.value)
-                           .select(rel.id, tmp.back_id), 'out(rel_id, back_id)')\
-                    .from_(out)\
-                    .select(out.rel_id, out.back_id)
+        query = Query.with_(Query.with_(Query.from_(ValuesTuple(*relations))
+                                        .select('*'), 'tmp(value, back_id)')
+                            .from_(tmp)
+                            .join(rel)
+                            .on(tmp.value == rel.value)
+                            .select(rel.id, tmp.back_id), 'out(rel_id, back_id)')\
+                     .from_(out)\
+                     .select(out.rel_id, out.back_id)
 
-    @staticmethod
-    def id_crit(ids, user, action):  # pylint: disable=unused-argument
-        dataset = Table('dataset')
-        return dataset.id.isin(ids)
+        ids = Table('ids')
+        empty = Query.select(ids.star).from_(ValuesTuple(Tuple(0, 0)).as_(ids)).where(NullValue())
 
-    @staticmethod
-    def setid_crit(ids, user, action):  # pylint: disable=unused-argument
+        return self.generate_crit(query, empty, user, action)
+
+    def id_crit(self, ids, user, action):
         dataset = Table('dataset')
-        return dataset.setid.isin([str(x) for x in ids])
+        tids = Table('tids')
+        ids = Query.from_(ValuesTuple(*[(x,) for x in ids]).as_('tids')).select(tids.star)
+        return dataset.id.isin(self.generate_crit(ids, Tuple(), user, action))
+
+    def setid_crit(self, ids, user, action):
+        dataset = Table('dataset')
+        tids = Table('tids')
+        ids = Query.from_(ValuesTuple(*[(str(x),) for x in ids]).as_('tids')).select(tids.star)
+        return dataset.setid.isin(self.generate_crit(ids, Tuple(), user, action))
 
     @run_in_transaction
     async def resolve_shortids(self, prefixes, discarded=False, txn=None):
@@ -1152,25 +1223,55 @@ class Database:
                      .orderby(tag.value)
         return [x['value'] for x in await txn.exq(query)]
 
+    @staticmethod
+    def get_listing_query(listing):
+        listing, dataset, dataset_tag, tag = \
+            Tables(listing, 'dataset', 'dataset_tag', 'tag')
+
+        return Query.from_(listing)\
+                    .join(dataset, how=JoinType.left_outer)\
+                    .on(listing.id == dataset.id)\
+                    .join(dataset_tag, how=JoinType.left_outer)\
+                    .on(dataset.id == dataset_tag.dataset_id)\
+                    .join(tag, how=JoinType.left_outer)\
+                    .on(dataset_tag.tag_id == tag.id)\
+                    .select(listing.id.as_('id'),
+                            listing.row.as_('row'),
+                            dataset.status.as_('status'),
+                            GroupConcat(tag.value).as_('tag_value'))
+
+    @staticmethod
+    def try_extended_filter(query, name, value, operator, val_type, col):  # pylint: disable=unused-argument
+        return False, query
+
+    @staticmethod
+    def postprocess_listing(rows):
+        def fmt(row):
+            if not row['tag_value']:
+                return '[]'
+            return json.dumps(sorted(row['tag_value'].split(',')))
+
+        return [
+            {
+                'id': row['id'],
+                'row': (row['row']
+                        .replace('["#TAGS#"]', fmt(row))
+                        .replace('"#TAGS#"', fmt(row)[1:-1] if fmt(row) != '[]' else '')
+                        .replace('[,', '[')
+                        .replace('"#STATUS#"', STATUS_STRS[row['status']])),
+            }
+            for row in rows
+        ]
+
     @run_in_transaction
-    async def get_filtered_listing(self, descs, filters, user, txn=None):  # noqa: C901
+    async def get_filtered_listing(self, descs, filters, collection, user, txn=None):  # noqa: C901
         # pylint: disable=too-many-locals,too-many-branches,too-many-statements,unused-argument
         listing, dataset, dataset_tag, tag = \
             Tables(descs[0].table, 'dataset', 'dataset_tag', 'tag')
 
-        query = Query.from_(listing)\
-                     .join(dataset, how=JoinType.left_outer)\
-                     .on(listing.id == dataset.id)\
-                     .join(dataset_tag, how=JoinType.left_outer)\
-                     .on(dataset.id == dataset_tag.dataset_id)\
-                     .join(tag, how=JoinType.left_outer)\
-                     .on(dataset_tag.tag_id == tag.id)\
-                     .select(listing.id.as_('id'),
-                             listing.row.as_('row'),
-                             dataset.status.as_('status'),
-                             GroupConcat(tag.value).as_('tag_value'))\
-                     .where(dataset.discarded.ne(True)
-                            & dataset.id.isin(self.get_actionable('dataset', user, 'list')))
+        query = self.get_listing_query(descs[0].table)\
+                    .where(dataset.discarded.ne(True)
+                           & dataset.id.isin(self.get_actionable('dataset', user, 'list')))
 
         for name, value, operator, val_type in filters:
             if isinstance(value, int):
@@ -1235,6 +1336,11 @@ class Database:
                 if operator in ['le', 'gt']:
                     value = value + 24 * 3600 * 1000 - 1
 
+            extended_done, query = self.try_extended_filter(query, name, value, operator, val_type,
+                                                            collection)
+            if extended_done:
+                continue
+
             field = getattr(listing, name)
             if operator == 'lt':
                 query = query.where(field < value)
@@ -1307,7 +1413,7 @@ class Database:
                 raise UnknownOperator(operator)
 
         query = query.orderby(tag.value).groupby('id')
-        return await txn.exq(query)
+        return self.postprocess_listing(await txn.exq(query))
 
     @run_in_transaction
     async def delete_listing_rel_values_without_ref(self, descs, txn=None):
@@ -1553,6 +1659,7 @@ class Database:
             for keys, handler in self.IMPORT_HANDLERS:
                 if keys.issubset(dct.keys()):
                     await handler(site, dct, txn)
+            dct.pop('version', None)
             if dct:
                 log.warning(
                     'Some fields from the dump file could not be imported. If you are imporing '
@@ -1583,7 +1690,7 @@ class Database:
                 connection = cls._connections['default']
                 return connection._in_transaction()  # pylint: disable=protected-access
 
-        dump = {}
+        dump = {'version': Database.DUMP_VERSION}
         async with await T2.transaction() as txn:
             master = Table('sqlite_master')
             tables = {
