@@ -14,7 +14,7 @@ from . import model_fields as custom
 from .utils import underscore_to_camelCase
 
 STATUS = OrderedDict((
-    ('error', 'ERROR: One or more errors occured while processing this dataset'),
+    ('error', 'ERROR: One or more errors occurred while processing this dataset'),
     ('missing', 'MISSING: One or more files of this dataset are missing'),
     ('outdated', 'OUTDATED: There are outdated nodes for this dataset'),
     ('pending', 'PENDING: There are pending node runs'),
@@ -84,6 +84,7 @@ class Dataset(Model):
     setid = custom.SetIDField(unique=True)
     tags = ManyToManyField('models.Tag', related_name='datasets')
     acn = ForeignKeyField('models.Acn', related_name='datasets', on_delete=RESTRICT)
+    dacn = ForeignKeyField('models.Acn', related_name='gdatasets', on_delete=RESTRICT)
 
     # Populated by backreferences: comments, files
 
@@ -139,6 +140,7 @@ class User(Model):
     email = TextField(null=True)
     realm = TextField()
     realmuid = TextField(null=True)
+    active = BooleanField()
     time_created = DatetimeField(auto_now_add=True)
     time_updated = DatetimeField(auto_now=True)
     groups = ManyToManyField('models.Group', related_name='users')
@@ -208,7 +210,8 @@ def make_listing_model(name, filter_specs):
 
     dct.update((fspec.name, coltype_factories[fspec.value_type](fspec.name))
                for fspec in filter_specs.values()
-               if fspec.name not in ('row', 'f_comments', 'f_status', 'f_tags'))
+               if fspec.name not in ('row', 'f_comments', 'f_status', 'f_tags')
+               if not fspec.name.startswith('f_leaf_'))
 
     models.append(type(listing_model_name, (Model,), dct))
     return models
