@@ -1,4 +1,4 @@
-# Copyright 2016 - 2018  Ternaris.
+# Copyright 2016 - 2021  Ternaris.
 # SPDX-License-Identifier: AGPL-3.0-only
 
 import json
@@ -9,12 +9,10 @@ from aiohttp import web
 
 from marv.collection import Filter
 from marv.db import DBPermissionError, UnknownOperator
-from marv.model import STATUS
 from marv.utils import parse_datetime, parse_filesize, parse_timedelta
 
-from .tooling import HTTPPermissionError
-from .tooling import api_endpoint as marv_api_endpoint
-from .tooling import get_global_granted, get_local_granted
+from .api import api
+from .tooling import HTTPPermissionError, get_global_granted, get_local_granted
 
 ALIGN = {
     'acceleration': 'right',
@@ -52,20 +50,6 @@ VALUE_TYPE_MAP = {
 TIMEZONE = pendulum.local_timezone().name
 
 
-# Order corresponds to marv.model.STATUS OrderedDict
-STATUS_ICON = ['fire', 'eye-close', 'warning-sign', 'time']
-STATUS_JSON = [jsondumps({'icon': STATUS_ICON[i], 'title': x},
-                         separators=(',', ':'))
-               for i, x in enumerate(STATUS.values())]
-# TODO: reconsider in case we get a couple of more states
-STATUS_STRS = {
-    bitmask: ','.join(filter(None,
-                             (STATUS_JSON[i] if bitmask & 2**i else None
-                              for i in range(len(STATUS_JSON)))))
-    for bitmask in range(2**(len(STATUS_JSON)))
-}
-
-
 def parse_filters(specs, filters):
     return [
         # TODO: remove adding of f_ prefix after listing deprecation
@@ -77,13 +61,10 @@ def parse_filters(specs, filters):
     ]
 
 
-@marv_api_endpoint('/meta', force_acl=['__unauthenticated__', '__authenticated__'], acl_key='list')
+@api.endpoint('/meta', methods=['GET'], allow_anon=True)
 async def meta(request):
     site = request.app['site']
-    if not set(request['user_groups']).intersection(request.app['route_acl']['list']):
-        collections = []
-    else:
-        collections = await site.db.get_collections(user=request['username'])
+    collections = await site.db.get_collections(user=request['username'])
 
     resp = web.json_response({
         'acl': get_global_granted(request),
@@ -95,7 +76,7 @@ async def meta(request):
     return resp
 
 
-@marv_api_endpoint('/collection{_:/?}{collection_id:((?<=/).*)?}', acl_key='read')
+@api.endpoint('/_collection{_:/?}{collection_id:((?<=/).*)?}', methods=['GET'], allow_anon=True)
 async def collection(request):  # pylint: disable=too-many-locals  # noqa: C901
     site = request.app['site']
     collection_id = request.match_info['collection_id'] or site.collections.default_id
@@ -120,7 +101,7 @@ async def collection(request):  # pylint: disable=too-many-locals  # noqa: C901
 
     try:
         rows = await site.db.get_filtered_listing(collection.table_descriptors, filters,
-                                                  user=request['username'])
+                                                  collection, user=request['username'])
     except (KeyError, ValueError, UnknownOperator):
         raise web.HTTPBadRequest()
 
@@ -143,16 +124,6 @@ async def collection(request):  # pylint: disable=too-many-locals  # noqa: C901
         for x in collection.filter_specs.values()
     ]
 
-    def fmt(row):
-        if not row['tag_value']:
-            return '[]'
-        return json.dumps(sorted(row['tag_value'].split(',')))
-
-    rowdata = ',\n'.join([row['row'].replace('["#TAGS#"]', fmt(row))
-                          .replace('"#TAGS#"', fmt(row)[1:-1] if fmt(row) != '[]' else '')
-                          .replace('[,', '[')
-                          .replace('"#STATUS#"', STATUS_STRS[row['status']])
-                          for row in rows])
     dct = {
         'acl': acl,
         'all_known': all_known,
@@ -183,7 +154,7 @@ async def collection(request):  # pylint: disable=too-many-locals  # noqa: C901
         indent = 2
         separators = (', ', ': ')
     jsondata = jsondumps(dct, indent=indent, separators=separators, sort_keys=True)
-    jsondata = jsondata.replace('"#ROWS#"', rowdata)
+    jsondata = jsondata.replace('"#ROWS#"', ',\n'.join(x['row'] for x in rows))
     return web.Response(text=jsondata, headers={
         'Content-Type': 'application/json',
         'Cache-Control': 'no-cache',
